<?php

namespace Drupal\big_datatable\Commands;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drush\Commands\DrushCommands;

/**
 * Class HtmlGeneratorCommands.
 *
 * Custom drush command to generate HTML from csv for big data table.
 *
 * @package Drupal\big_datatable
 */
class HtmlGeneratorCommands extends DrushCommands {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  public function __construct(EntityTypeManagerInterface $entityTypeManager, FileSystemInterface $file_system) {
    $this->entityTypeManager = $entityTypeManager;
    $this->fileSystem = $file_system;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('file_system')
    );
  }

  /**
   * Get all big data table entities.
   *
   * @command big_datatable:get-big-data-table-entities
   * @aliases bigdata-generate-html
   * @usage big_datatable:get-big-data-table-entities
   */
  public function getBigDataTableEntities() {
    $storage = $this->entityTypeManager->getStorage('big_datatable');
    $entities = $storage->loadMultiple();

    foreach ($entities as $entity) {
      $file = $this->entityTypeManager->getStorage('file')->load($entity->getBigdataFile());
      $csv_file_path = $file->getFileUri();
      $generated_html = $entity->id();
      if (($handle = fopen($csv_file_path, 'r')) !== FALSE) {
        $csv_data = [];
        while (($row = fgetcsv($handle, 1000, ',')) !== FALSE) {
          $csv_data[] = $row;
        }
        fclose($handle);
        $html_content = $this->generateHtmlFromCsv($csv_data);
        $html_file_path = $this->fileSystem->realpath('public://big_data_tables/' . $generated_html . '.html');
        file_put_contents($html_file_path, $html_content);
      }

    }
  }

  /**
   * Generates HTML content from CSV data.
   *
   * @param array $csv_data
   *   The CSV data.
   *
   * @return string
   *   The generated HTML content.
   */
  protected function generateHtmlFromCsv($csv_data) {
    $html = '<table id="maintable" class="display compact cell-border" cellspacing="0" width="100%">';
    if (!empty($csv_data)) {
      $header = array_shift($csv_data);
      $html .= '<thead><tr>';
      foreach ($header as $cell) {
        $html .= '<th>' . htmlspecialchars($cell) . '</th>';
      }
      $html .= '</tr></thead>';
    }
    $html .= '<tbody>';
    foreach ($csv_data as $row) {
      $html .= '<tr>';
      foreach ($row as $cell) {
        $html .= '<td>' . htmlspecialchars($cell) . '</td>';
      }
      $html .= '</tr>';
    }
    $html .= '</tbody>';
    if (!empty($header)) {
      $html .= '<tfoot><tr>';
      foreach ($header as $cell) {
        $html .= '<th>' . htmlspecialchars($cell) . '</th>';
      }
      $html .= '</tr></tfoot>';
    }

    $html .= '</table>';

    return $html;
  }

}
