<?php

namespace Drupal\big_datatable\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\file\FileUsage\FileUsageInterface;
use Drupal\path_alias\AliasRepositoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Big Data Table form.
 *
 * @property \Drupal\big_datatable\BigDataTableInterface $entity
 */
class BigDataTableForm extends EntityForm {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The file usage service.
   *
   * @var \Drupal\file\FileUsage\FileUsageInterface
   */
  protected $fileUsage;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The path alias repository.
   *
   * @var \Drupal\path_alias\AliasRepositoryInterface
   */
  protected $patAliasRepository;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    EntityTypeManagerInterface $entityTypeManager,
    FileUsageInterface $fileUsage,
    RequestStack $request_stack,
    LanguageManagerInterface $language_manager,
    AliasRepositoryInterface $path_alias_epository,
    MessengerInterface $messenger,
    FileSystemInterface $file_system,
  ) {
    $this->entityTypeManager = $entityTypeManager;
    $this->fileUsage = $fileUsage;
    $this->requestStack = $request_stack;
    $this->languageManager = $language_manager;
    $this->patAliasRepository = $path_alias_epository;
    $this->messenger = $messenger;
    $this->fileSystem = $file_system;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('file.usage'),
      $container->get('request_stack'),
      $container->get('language_manager'),
      $container->get('path_alias.repository'),
      $container->get('messenger'),
      $container->get('file_system')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);
    global $base_url;
    $lang_code = $this->languageManager->getCurrentLanguage()->getId();
    $form['name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Name'),
      '#maxlength' => 255,
      '#default_value' => $this->entity->label(),
      '#description' => $this->t("Label for the Big Data Table"),
      '#required' => TRUE,
    ];
    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $this->entity->id(),
      '#machine_name' => [
        'exists' => [$this, 'exist'],
        'source' => ['name'],
      ],
      '#disabled' => !$this->entity->isNew(),
    ];
    $form['title'] = [
      '#type' => 'textfield',
      '#title' => $this->t("Table's page title"),
      '#default_value' => $this->entity->getTitle(),
      '#required' => TRUE,
    ];
    $body = $this->entity->getBody();
    $form['body'] = [
      '#type' => 'text_format',
      '#title' => $this->t("Table's page description"),
      '#default_value' => $body && isset($body['value']) ? $body['value'] : '',
      '#format' => $body && isset($body['format']) ? $body['format'] : NULL,
    ];
    $form['bigdata_file'] = [
      '#type' => 'managed_file',
      '#title' => $this->t('File'),
      '#upload_location' => 'public://big_data_tables/',
      '#default_value' => [$this->entity->getBigdataFile() ?? ""],
      '#description' => $this->t('Upload CSV file to render the data on page.'),
      '#upload_validators'  => [
        'file_validate_extensions' => ['csv'],
      ],
      '#required' => TRUE,
    ];

    $alias = "";
    if (!$this->entity->isNew()) {
      $alias = $this->patAliasRepository->lookupBySystemPath('/big-data-table/' . $this->entity->Id(), $lang_code)['alias'];
    }
    $form['alias'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Page alias or Path'),
      '#default_value' => $alias,
      '#description' => $this->t('Specify an alternative path by which this data can be accessed. For example, type "/about" when writing an about page.'),
      '#field_prefix' => $base_url,
      '#required' => TRUE,
    ];
    if (!$this->entity->isNew() && $this->entity->getBigdataFile() > 0) {
      $form['submit_button_one'] = [
        '#type' => 'submit',
        '#value' => $this->t('Generate HTML'),
        '#submit' => ['::generateHtml'],
      ];
    }
    return $form;
  }

  /**
   * Submit handler for the generateHtml.
   */
  public function generateHtml(array &$form, FormStateInterface $form_state) {
    $file_id = $form_state->getValue('bigdata_file')[0];
    if ($file_id) {
      $file = $this->entityTypeManager->getStorage('file')->load($file_id);
      if ($file) {
        $generated_html = $form_state->getValue('id');
        $csv_file_path = $file->getFileUri();
        if (file_exists($csv_file_path)) {
          $operations = [
            [
              [$this, 'processCsvToHtml'],
              [$csv_file_path, $generated_html],
            ],
          ];
          $batch = [
            'title' => $this->t('Generating HTML'),
            'operations' => $operations,
            'finished' => [$this, 'batchFinished'],
          ];
          batch_set($batch);
        }
        else {
          $this->messenger->addError($this->t('CSV file not found.'));
        }
      }
      else {
        $this->messenger->addError($this->t('File entity could not be loaded.'));
      }
    }
    else {
      $this->messenger->addError($this->t('No file uploaded.'));
    }
  }

  /**
   * Batch operation callback to process CSV and generate HTML.
   */
  public function processCsvToHtml($csv_file_path, $generated_html, &$context) {
    if (($handle = fopen($csv_file_path, 'r')) !== FALSE) {
      $csv_data = [];
      while (($row = fgetcsv($handle, 1000, ',')) !== FALSE) {
        $csv_data[] = $row;
      }
      fclose($handle);
      $html_content = $this->generateHtmlFromCsv($csv_data);
      $html_file_path = $this->fileSystem->realpath('public://big_data_tables/' . $generated_html . '.html');
      file_put_contents($html_file_path, $html_content);
      $context['results']['html_file_path'] = $html_file_path;
    }
  }

  /**
   * Batch finished callback.
   */
  public function batchFinished($success, $results, $operations) {
    if ($success) {
      $this->messenger->addMessage($this->t('HTML file generated successfully at @path.', ['@path' => $results['html_file_path']]));
    }
    else {
      $this->messenger->addError($this->t('An error occurred during HTML generation.'));
    }
  }

  /**
   * Generates HTML content from CSV data.
   *
   * @param array $csv_data
   *   The CSV data.
   *
   * @return string
   *   The generated HTML content.
   */
  protected function generateHtmlFromCsv(array $csv_data) {
    $html = '<table id="maintable" class="display compact cell-border" cellspacing="0" width="100%">';
    if (!empty($csv_data)) {
      $header = array_shift($csv_data);
      $html .= '<thead><tr>';
      foreach ($header as $cell) {
        $html .= '<th>' . htmlspecialchars($cell) . '</th>';
      }
      $html .= '</tr></thead>';
    }
    $html .= '<tbody>';
    foreach ($csv_data as $row) {
      $html .= '<tr>';
      foreach ($row as $cell) {
        $html .= '<td>' . htmlspecialchars($cell) . '</td>';
      }
      $html .= '</tr>';
    }
    $html .= '</tbody>';
    if (!empty($header)) {
      $html .= '<tfoot><tr>';
      foreach ($header as $cell) {
        $html .= '<th>' . htmlspecialchars($cell) . '</th>';
      }
      $html .= '</tr></tfoot>';
    }

    $html .= '</table>';

    return $html;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
    $lang_code = $this->languageManager->getCurrentLanguage()->getId();
    $alias = $form_state->getValue('alias');
    if (!preg_match('/^\//i', $alias)) {
      $form_state->setErrorByName('alias', $this->t('The alias path has to start with a slash.'));
    }

    $lookupByAlias = $this->patAliasRepository->lookupByAlias($alias, $lang_code);
    $lookupByPath = $this->patAliasRepository->lookupBySystemPath('/big-data-table/' . $this->entity->Id(), $lang_code);

    if (($this->entity->isNew() && $lookupByAlias) || ($lookupByAlias && $lookupByPath['alias'] != $alias)) {
      $form_state->setErrorByName('alias', $this->t('The alias @alias is already in use in this language', ['@alias' => $alias]));
    }

  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $edit_redirect_flag = FALSE;
    $this->entity->setBigdataFile($form_state->getValue('bigdata_file'));
    $file = $this->entityTypeManager->getStorage('file')->load($this->entity->getBigdataFile());

    if (!$file->isPermanent()) {
      $file->setPermanent();
      $file->save();
      $this->fileUsage->add($file, 'big_datatable', 'big_datatable', $this->entity->id());

      // Deleting old file and entry from the database.
      if (!$this->entity->isNew()) {
        $oldFileId = $this->currentEntity($this->entity->id())->getBigdataFile();
        if ($oldFileId > 0 &&  $file->id() > 0 && $oldFileId != $file->id()) {
          $oldFile = $this->entityTypeManager->getStorage('file')->load($oldFileId);
          $oldFile->delete();
        }
      }
      $this->messenger()->addStatus($this->t('File updated successfully.'));

      $edit_redirect_flag = TRUE;
    }

    $status = parent::save($form, $form_state);

    if ($status) {
      $this->tablePagePath($this->entity);
      $this->messenger()->addStatus($this->t('Saved the %label Big Data Table.', [
        '%label' => $this->entity->label(),
      ]));
    }
    else {
      $edit_redirect_flag = FALSE;
      $this->messenger()->addError($this->t('The %label Big Datae Table was not saved.', [
        '%label' => $this->entity->label(),
      ]));
    }

    $form_state->setRedirect('entity.big_datatable.collection');
    if ($edit_redirect_flag) {
      $this->requestStack->getCurrentRequest()->query->remove('destination');
      $form_state->setRedirect('entity.big_datatable.edit_form', ['big_datatable' => $this->entity->id()]);
    }

  }

  /**
   * Helper function to check whether an big_datatable.
   *
   * Configuration entity exists.
   */
  public function exist($id) {
    $entity = $this->entityTypeManager
      ->getStorage('big_datatable')
      ->getQuery()
      ->condition('id', $id)
      ->execute();
    return (bool) $entity;
  }

  /**
   * Helper function to check whether an big_datatable.
   *
   * Load current entity.
   */
  public function currentEntity($id) {
    $entity = $this->entityTypeManager->getStorage('big_datatable')->load($id);
    return $entity;
  }

  /**
   * Helper function to create or update the path of the table page.
   */
  public function tablePagePath($big_data_table) {
    if ($big_data_table->Id()) {
      $path = '/big-data-table/' . $this->entity->Id();
      $alias = $big_data_table->alias;
      $lang_code = $this->languageManager->getCurrentLanguage()->getId();
      $lookupByPath = $this->patAliasRepository->lookupBySystemPath($path, $lang_code);
      $alias_storage = $this->entityTypeManager->getStorage('path_alias');
      $alias_object = "";
      if (!$lookupByPath) {
        $alias_object = $alias_storage->create([
          'path' => '/big-data-table/' . $this->entity->Id(),
          'alias' => $alias,
          'langcode' => $lang_code,
        ]);
      }
      elseif ($lookupByPath && $lookupByPath['alias'] != $alias) {
        $alias_object = $alias_storage->load($lookupByPath['id']);
        $alias_object->setAlias($alias);
      }

      if ($alias_object) {
        $path_status = $alias_object->save();
        if ($path_status) {
          $this->messenger()->addStatus($this->t('Alias updated of the %label Big Data Table.', [
            '%label' => $this->entity->label(),
          ]));
        }
        else {
          $this->messenger()->addError($this->t('Alias of the %label Big Data Table was not saved.', [
            '%label' => $this->entity->label(),
          ]));
        }

      }

    }
    return FALSE;
  }

}
